@@ -7,4 +7,4 @@ static inline struct pthread *__pthread_self()
 
 #define TP_ADJ(p) (p)
 
-#define CANCEL_REG_IP 16
+#define MC_PC gregs[REG_RIP]
