@@ -1,5 +1,3 @@
-#ifdef SHARED
-
 #include <stddef.h>
 #include "libc.h"
 
@@ -12,5 +10,3 @@ ptrdiff_t __tlsdesc_static()
 }
 
 weak_alias(__tlsdesc_static, __tlsdesc_dynamic);
-
-#endif
