@@ -1,6 +1,6 @@
+#define _GNU_SOURCE
 #include <string.h>
-#include <fcntl.h>
-#include <unistd.h>
+#include <stdlib.h>
 #include <errno.h>
 #include <sys/stat.h>
 
